package xhttp3.internal.cache;

import java.io.IOException;
import java.util.Date;
import java.util.concurrent.TimeUnit;
import xhttp3.Headers;
import xhttp3.Interceptor;
import xhttp3.MediaType;
import xhttp3.Protocol;
import xhttp3.Request;
import xhttp3.Response;
import xhttp3.ResponseBody;
import xhttp3.internal.Internal;
import xhttp3.internal.Util;
import xhttp3.internal.http.HttpHeaders;
import xhttp3.internal.http.HttpMethod;
import xhttp3.internal.http.RealResponseBody;
import xio.Buffer;
import xio.BufferedSink;
import xio.BufferedSource;
import xio.Okio;
import xio.Sink;
import xio.Source;
import xio.Timeout;

public final class CacheInterceptor implements Interceptor {
   private static final ResponseBody EMPTY_BODY = new ResponseBody() {
      public MediaType contentType() {
         return null;
      }

      public long contentLength() {
         return 0L;
      }

      public BufferedSource source() {
         return new Buffer();
      }
   };
   final InternalCache cache;

   public CacheInterceptor(InternalCache cache) {
      this.cache = cache;
   }

   private static Response stripBody(Response response) {
      if (response != null && response.body() != null) {
         response = response.newBuilder().body((ResponseBody)null).build();
      }

      return response;
   }

   private CacheRequest maybeCache(Response userResponse, Request networkRequest, InternalCache responseCache) {
      if (responseCache == null) {
         return null;
      } else if (!CacheStrategy.isCacheable(userResponse, networkRequest)) {
         if (HttpMethod.invalidatesCache(networkRequest.method())) {
            try {
               responseCache.remove(networkRequest);
            } catch (IOException var4) {
            }
         }

         return null;
      } else {
         return responseCache.put(userResponse);
      }
   }

   private Response cacheWritingResponse(CacheRequest cacheRequest, Response response) {
      if (cacheRequest == null) {
         return response;
      } else {
         Sink var3;
         if ((var3 = cacheRequest.body()) == null) {
            return response;
         } else {
            Sink var10001 = var3;
            BufferedSource var7 = response.body().source();
            BufferedSink var4 = Okio.buffer(var10001);
            Source var5;
            var5 = new Source() {
               boolean cacheRequestClosed;
               // $FF: synthetic field
               final BufferedSource val$source;
               // $FF: synthetic field
               final CacheRequest val$cacheRequest;
               // $FF: synthetic field
               final BufferedSink val$cacheBody;

               {
                  this.val$source = var2;
                  this.val$cacheRequest = var3;
                  this.val$cacheBody = var4;
               }

               public long read(Buffer sink, long byteCount) {
                  long var10000;
                  try {
                     var10000 = this.val$source.read(sink, byteCount);
                  } catch (IOException var4) {
                     if (!this.cacheRequestClosed) {
                        this.cacheRequestClosed = true;
                        this.val$cacheRequest.abort();
                     }

                     throw var4;
                  }

                  byteCount = var10000;
                  if (var10000 == -1L) {
                     if (!this.cacheRequestClosed) {
                        this.cacheRequestClosed = true;
                        this.val$cacheBody.close();
                     }

                     return -1L;
                  } else {
                     sink.copyTo(this.val$cacheBody.buffer(), sink.size() - byteCount, byteCount);
                     this.val$cacheBody.emitCompleteSegments();
                     return byteCount;
                  }
               }

               public Timeout timeout() {
                  return this.val$source.timeout();
               }

               public void close() {
                  if (!this.cacheRequestClosed && !Util.discard(this, 100, TimeUnit.MILLISECONDS)) {
                     this.cacheRequestClosed = true;
                     this.val$cacheRequest.abort();
                  }

                  this.val$source.close();
               }
            }.<init>(var7, cacheRequest, var4);
            Response.Builder var10000 = response.newBuilder();
            Headers var6 = response.headers();
            return var10000.body(new RealResponseBody(var6, Okio.buffer(var5))).build();
         }
      }
   }

   private static boolean validate(Response cached, Response network) {
      if (network.code() == 304) {
         return true;
      } else {
         Date cached1;
         Date network1;
         return (cached1 = cached.headers().getDate("Last-Modified")) != null && (network1 = network.headers().getDate("Last-Modified")) != null && network1.getTime() < cached1.getTime();
      }
   }

   private static Headers combine(Headers cachedHeaders, Headers networkHeaders) {
      Headers.Builder var2;
      var2 = new Headers.Builder.<init>();
      int var3 = 0;

      for(int var4 = cachedHeaders.size(); var3 < var4; ++var3) {
         String var5 = cachedHeaders.name(var3);
         String var6 = cachedHeaders.value(var3);
         if ((!"Warning".equalsIgnoreCase(var5) || !var6.startsWith("1")) && (!isEndToEnd(var5) || networkHeaders.get(var5) == null)) {
            Internal.instance.addLenient(var2, var5, var6);
         }
      }

      int cachedHeaders1 = 0;

      for(var3 = networkHeaders.size(); cachedHeaders1 < var3; ++cachedHeaders1) {
         String var8 = networkHeaders.name(cachedHeaders1);
         if (!"Content-Length".equalsIgnoreCase(var8) && isEndToEnd(var8)) {
            Internal.instance.addLenient(var2, var8, networkHeaders.value(cachedHeaders1));
         }
      }

      return var2.build();
   }

   static boolean isEndToEnd(String fieldName) {
      return !"Connection".equalsIgnoreCase(fieldName) && !"Keep-Alive".equalsIgnoreCase(fieldName) && !"Proxy-Authenticate".equalsIgnoreCase(fieldName) && !"Proxy-Authorization".equalsIgnoreCase(fieldName) && !"TE".equalsIgnoreCase(fieldName) && !"Trailers".equalsIgnoreCase(fieldName) && !"Transfer-Encoding".equalsIgnoreCase(fieldName) && !"Upgrade".equalsIgnoreCase(fieldName);
   }

   public Response intercept(Interceptor.Chain chain) {
      InternalCache var2;
      Response var12;
      if ((var2 = this.cache) != null) {
         var12 = var2.get(chain.request());
      } else {
         var12 = null;
      }

      long var3 = System.currentTimeMillis();
      Request var5 = chain.request();
      CacheStrategy var13;
      CacheStrategy var10001 = var13 = (new CacheStrategy.Factory(var3, var5, var12)).get();
      Request var4 = var10001.networkRequest;
      Response var14 = var10001.cacheResponse;
      InternalCache var6;
      if ((var6 = this.cache) != null) {
         var6.trackResponse(var13);
      }

      if (var12 != null && var14 == null) {
         Util.closeQuietly(var12.body());
      }

      if (var4 == null && var14 == null) {
         return (new Response.Builder()).request(chain.request()).protocol(Protocol.HTTP_1_1).code(504).message("Unsatisfiable Request (only-if-cached)").body(EMPTY_BODY).sentRequestAtMillis(-1L).receivedResponseAtMillis(System.currentTimeMillis()).build();
      } else if (var4 == null) {
         return var14.newBuilder().cacheResponse(stripBody(var14)).build();
      } else {
         boolean var8 = false;

         Response var10000;
         try {
            var8 = true;
            var10000 = chain.proceed(var4);
            var8 = false;
         } finally {
            if (var8) {
               if (var12 != null) {
                  Util.closeQuietly(var12.body());
               }

            }
         }

         Response chain1 = var10000;
         if (var10000 == null && var12 != null) {
            Util.closeQuietly(var12.body());
         }

         if (var14 != null) {
            if (validate(var14, chain1)) {
               var10000 = var12 = var14.newBuilder().headers(combine(var14.headers(), chain1.headers())).cacheResponse(stripBody(var14)).networkResponse(stripBody(chain1)).build();
               chain1.body().close();
               this.cache.trackConditionalCacheHit();
               this.cache.update(var14, var12);
               return var10000;
            }

            Util.closeQuietly(var14.body());
         }

         if (HttpHeaders.hasBody(var12 = chain1.newBuilder().cacheResponse(stripBody(var14)).networkResponse(stripBody(chain1)).build())) {
            CacheInterceptor var15 = this;
            CacheInterceptor var16 = this;
            CacheInterceptor var10003 = this;
            Request var10 = chain1.request();
            var12 = var15.cacheWritingResponse(var16.maybeCache(var12, var10, var10003.cache), var12);
         }

         return var12;
      }
   }
}
